"""
Flask-Hooker
--------------
"""

from setuptools import setup

setup(
    name='Flask-Hooker',
    version="1.0.1",
    url='http://github.com/doblel/Flask-Hooker/',
    license='MIT',
    author='Lisandro Lucatti (doblel)',
    author_email='lucattilisandro@gmail.com',
    description='Receive and manage webhooks of several services at the same time',
    long_description='',
    packages=['flask_hooker'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={},
    install_requires=['Flask'],
    tests_require=[],
    test_suite='t',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



"""Core."""
from flask import Blueprint, jsonify, request


class Hooker(object):
    """Docs here."""

    def __init__(self, app=None, name='hooker', url_prefix='/webhooks', methods=['GET', 'POST']):
        """Create the Hooker object."""
        self._app = app
        self._name = name
        self._methods = methods
        self._blueprint = Blueprint(name, __name__, url_prefix=url_prefix)
        self._event_handler = {}

        if app:
            self.init_app(app)

    def add_handler(self, event=None, func=None, event_type=None):
        """Add function to handle a specific event from an event_type."""
        if event and func and event_type:
            new_event = {event: func}

            if event_type not in self._event_handler:
                self._event_handler[event_type] = {}

            self._event_handler[event_type].update(new_event)
        else:
            print 'All parameters are required.'

    def _run_handlers(self):
        if request.method == 'GET':
            return 'OK'
        for event_type in self._event_handler.keys():
            if event_type in request.headers and request.headers.get("Content-Type", '') == "application/json":
                try:
                    event = request.headers.get(event_type)
                    if event in self._event_handler[event_type].keys():
                        self._event_handler[event_type][event](request.json)
                    else:
                        print 'event not allowed.'

                except Exception as e:
                    print e

                return jsonify({
                    'status': 'success',
                    'message': 'Event: %s, was triggered successfully' % event
                })
            else:
                pass

    def init_app(self, app):
        """Register the webhook."""
        self._blueprint.add_url_rule('', view_func=self._run_handlers, methods=self._methods)
        app.register_blueprint(self._blueprint)
